@@ -102,11 +102,6 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-elisp-src",
             extra_files: vec![],
         },
-        TreeSitterParser {
-            name: "tree-sitter-elixir",
-            src_dir: "vendored_parsers/tree-sitter-elixir-src",
-            extra_files: vec!["scanner.c"],
-        },
         TreeSitterParser {
             name: "tree-sitter-elm",
             src_dir: "vendored_parsers/tree-sitter-elm-src",
@@ -117,16 +112,6 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-elvish-src",
             extra_files: vec![],
         },
-        TreeSitterParser {
-            name: "tree-sitter-erlang",
-            src_dir: "vendored_parsers/tree-sitter-erlang-src",
-            extra_files: vec![],
-        },
-        TreeSitterParser {
-            name: "tree-sitter-f-sharp",
-            src_dir: "vendored_parsers/tree-sitter-f-sharp-src",
-            extra_files: vec!["scanner.c"],
-        },
         TreeSitterParser {
             name: "tree-sitter-gleam",
             src_dir: "vendored_parsers/tree-sitter-gleam-src",
@@ -162,26 +147,11 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-latex-src",
             extra_files: vec!["scanner.c"],
         },
-        TreeSitterParser {
-            name: "tree-sitter-make",
-            src_dir: "vendored_parsers/tree-sitter-make-src",
-            extra_files: vec![],
-        },
         TreeSitterParser {
             name: "tree-sitter-newick",
             src_dir: "vendored_parsers/tree-sitter-newick-src",
             extra_files: vec![],
         },
-        TreeSitterParser {
-            name: "tree-sitter-nix",
-            src_dir: "vendored_parsers/tree-sitter-nix-src",
-            extra_files: vec!["scanner.c"],
-        },
-        TreeSitterParser {
-            name: "tree-sitter-pascal",
-            src_dir: "vendored_parsers/tree-sitter-pascal-src",
-            extra_files: vec![],
-        },
         TreeSitterParser {
             name: "tree-sitter-perl",
             src_dir: "vendored_parsers/tree-sitter-perl-src",
@@ -202,11 +172,6 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-racket-src",
             extra_files: vec!["scanner.c"],
         },
-        TreeSitterParser {
-            name: "tree-sitter-rust",
-            src_dir: "vendored_parsers/tree-sitter-rust-src",
-            extra_files: vec!["scanner.c"],
-        },
         TreeSitterParser {
             name: "tree-sitter-scheme",
             src_dir: "vendored_parsers/tree-sitter-scheme-src",
@@ -237,26 +202,11 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-sql-src",
             extra_files: vec!["scanner.cc"],
         },
-        TreeSitterParser {
-            name: "tree-sitter-swift",
-            src_dir: "vendored_parsers/tree-sitter-swift-src",
-            extra_files: vec!["scanner.c"],
-        },
         TreeSitterParser {
             name: "tree-sitter-vhdl",
             src_dir: "vendored_parsers/tree-sitter-vhdl-src",
             extra_files: vec![],
         },
-        TreeSitterParser {
-            name: "tree-sitter-yaml",
-            src_dir: "vendored_parsers/tree-sitter-yaml-src",
-            extra_files: vec!["scanner.cc"],
-        },
-        TreeSitterParser {
-            name: "tree-sitter-zig",
-            src_dir: "vendored_parsers/tree-sitter-zig-src",
-            extra_files: vec![],
-        },
     ];
 
     // Only rerun if relevant files in the vendored_parsers/ directory change.
@@ -270,6 +220,14 @@ fn main() {
     if let Some((version, _, _)) = rustc::triple() {
         println!("cargo:rustc-env=DFT_RUSTC_VERSION={}", version);
     }
+
+    // Use 64-KiB pages with jemalloc. This solves "<jemalloc>:
+    // Unsupported system page size" errors, and performs the same as
+    // jemalloc's default settings.
+    //
+    // Note that difftastic does not use jemalloc on all operating
+    // systems, but it's harmless to set this unconditionally.
+    println!("cargo:rustc-env=JEMALLOC_SYS_WITH_LG_PAGE=16");
 }
 
 fn commit_info() {
