@@ -4,7 +4,6 @@
 use std::{cmp::Reverse, env};
 
 use bumpalo::Bump;
-use itertools::Itertools;
 use radix_heap::RadixHeapMap;
 
 use crate::{
@@ -240,7 +239,7 @@ pub(crate) fn mark_syntax<'a>(
                 )
             })
             .take(print_length)
-            .collect_vec()
+            .collect::<Vec<_>>()
     );
 
     populate_change_map(&route, change_map);
@@ -249,7 +248,6 @@ pub(crate) fn mark_syntax<'a>(
 
 #[cfg(test)]
 mod tests {
-    use itertools::Itertools;
     use line_numbers::SingleLineSpan;
     use typed_arena::Arena;
 
@@ -273,16 +271,16 @@ mod tests {
     fn identical_atoms() {
         let arena = Arena::new();
 
-        let lhs = Syntax::new_atom(&arena, pos_helper(0), "foo", AtomKind::Normal);
+        let lhs = Syntax::new_atom(&arena, pos_helper(0), "foo".to_owned(), AtomKind::Normal);
         // Same content as LHS.
-        let rhs = Syntax::new_atom(&arena, pos_helper(0), "foo", AtomKind::Normal);
+        let rhs = Syntax::new_atom(&arena, pos_helper(0), "foo".to_owned(), AtomKind::Normal);
         init_all_info(&[lhs], &[rhs]);
 
         let start = Vertex::new(Some(lhs), Some(rhs));
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![UnchangedNode {
@@ -303,7 +301,7 @@ mod tests {
             vec![Syntax::new_atom(
                 &arena,
                 pos_helper(1),
-                "foo",
+                "foo".to_owned(),
                 AtomKind::Normal,
             )],
             "]",
@@ -320,11 +318,11 @@ mod tests {
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
+        let start = Vertex::new(lhs.first().copied(), rhs.first().copied());
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![
@@ -354,19 +352,19 @@ mod tests {
             "[",
             pos_helper(0),
             vec![
-                Syntax::new_atom(&arena, pos_helper(1), "foo", AtomKind::Normal),
-                Syntax::new_atom(&arena, pos_helper(2), "foo", AtomKind::Normal),
+                Syntax::new_atom(&arena, pos_helper(1), "foo".to_owned(), AtomKind::Normal),
+                Syntax::new_atom(&arena, pos_helper(2), "foo".to_owned(), AtomKind::Normal),
             ],
             "]",
             pos_helper(3),
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
+        let start = Vertex::new(lhs.first().copied(), rhs.first().copied());
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![
@@ -389,7 +387,7 @@ mod tests {
             pos_helper(0),
             vec![
                 Syntax::new_list(&arena, "(", pos_helper(1), vec![], ")", pos_helper(2)),
-                Syntax::new_atom(&arena, pos_helper(3), "foo", AtomKind::Normal),
+                Syntax::new_atom(&arena, pos_helper(3), "foo".to_owned(), AtomKind::Normal),
             ],
             "]",
             pos_helper(4),
@@ -401,18 +399,18 @@ mod tests {
             pos_helper(0),
             vec![
                 Syntax::new_list(&arena, "(", pos_helper(1), vec![], ")", pos_helper(2)),
-                Syntax::new_atom(&arena, pos_helper(3), "foo", AtomKind::Normal),
+                Syntax::new_atom(&arena, pos_helper(3), "foo".to_owned(), AtomKind::Normal),
             ],
             "}",
             pos_helper(4),
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
+        let start = Vertex::new(lhs.first().copied(), rhs.first().copied());
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![
@@ -437,23 +435,23 @@ mod tests {
         let lhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "the quick brown fox",
+            "the quick brown fox".to_owned(),
             AtomKind::Comment,
         )];
 
         let rhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "the quick brown cat",
+            "the quick brown cat".to_owned(),
             AtomKind::Comment,
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
+        let start = Vertex::new(lhs.first().copied(), rhs.first().copied());
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![ReplacedComment {
@@ -469,23 +467,23 @@ mod tests {
         let lhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "the quick brown fox",
+            "the quick brown fox".to_owned(),
             AtomKind::Comment,
         )];
 
         let rhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "foo bar",
+            "foo bar".to_owned(),
             AtomKind::Comment,
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
+        let start = Vertex::new(lhs.first().copied(), rhs.first().copied());
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![ReplacedComment {
@@ -502,13 +500,13 @@ mod tests {
             Syntax::new_atom(
                 &arena,
                 pos_helper(1),
-                "the quick brown fox",
+                "the quick brown fox".to_owned(),
                 AtomKind::Comment,
             ),
             Syntax::new_atom(
                 &arena,
                 pos_helper(2),
-                "the quick brown thing",
+                "the quick brown thing".to_owned(),
                 AtomKind::Comment,
             ),
         ];
@@ -516,16 +514,16 @@ mod tests {
         let rhs = vec![Syntax::new_atom(
             &arena,
             pos_helper(1),
-            "the quick brown fox.",
+            "the quick brown fox.".to_owned(),
             AtomKind::Comment,
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
+        let start = Vertex::new(lhs.first().copied(), rhs.first().copied());
         let vertex_arena = Bump::new();
         let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
 
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
+        let actions = route.iter().map(|(action, _)| *action).collect::<Vec<_>>();
         assert_eq!(
             actions,
             vec![
@@ -540,8 +538,8 @@ mod tests {
     #[test]
     fn mark_syntax_equal_atoms() {
         let arena = Arena::new();
-        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo", AtomKind::Normal);
-        let rhs = Syntax::new_atom(&arena, pos_helper(1), "foo", AtomKind::Normal);
+        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo".to_owned(), AtomKind::Normal);
+        let rhs = Syntax::new_atom(&arena, pos_helper(1), "foo".to_owned(), AtomKind::Normal);
         init_all_info(&[lhs], &[rhs]);
 
         let mut change_map = ChangeMap::default();
@@ -554,8 +552,8 @@ mod tests {
     #[test]
     fn mark_syntax_different_atoms() {
         let arena = Arena::new();
-        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo", AtomKind::Normal);
-        let rhs = Syntax::new_atom(&arena, pos_helper(1), "bar", AtomKind::Normal);
+        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo".to_owned(), AtomKind::Normal);
+        let rhs = Syntax::new_atom(&arena, pos_helper(1), "bar".to_owned(), AtomKind::Normal);
         init_all_info(&[lhs], &[rhs]);
 
         let mut change_map = ChangeMap::default();
