@@ -19,9 +19,14 @@
 // implementation does not consider the mutable fields, so it is still
 // correct.
 #![allow(clippy::mutable_key_type)]
+// manual_unwrap_or_default was added in Rust 1.79, so earlier versions of
+// clippy complain about allowing it.
+#![allow(unknown_lints)]
 // It's sometimes more readable to explicitly create a vec than to use
 // the Default trait.
 #![allow(clippy::manual_unwrap_or_default)]
+// I find the explicit arithmetic clearer sometimes.
+#![allow(clippy::implicit_saturating_sub)]
 // .to_owned() is more explicit on string references.
 #![warn(clippy::str_to_string)]
 // .to_string() on a String is clearer as .clone().
@@ -50,7 +55,6 @@ extern crate log;
 
 use display::style::print_warning;
 use log::info;
-use mimalloc::MiMalloc;
 use options::FilePermissions;
 use options::USAGE;
 
@@ -73,15 +77,31 @@ use crate::parse::syntax;
 
 /// The global allocator used by difftastic.
 ///
-/// Diffing allocates a large amount of memory, and `MiMalloc` performs
-/// better.
+/// Diffing allocates a large amount of memory, and both Jemalloc and
+/// MiMalloc perform better than the system allocator.
+///
+/// Some versions of MiMalloc (specifically libmimalloc-sys greater
+/// than 0.1.24) handle very large, mostly unused allocations
+/// badly. This makes large line-oriented diffs very slow, as
+/// discussed in #297.
+///
+/// MiMalloc is generally faster than Jemalloc, but older versions of
+/// MiMalloc don't compile on GCC 15+, so use Jemalloc for now. See
+/// #805.
+///
+/// For reference, Jemalloc uses 10-20% more time (although up to 33%
+/// more instructions) when testing on sample files.
+#[cfg(not(any(target_env = "msvc", target_os = "illumos")))]
+use tikv_jemallocator::Jemalloc;
+
+#[cfg(not(any(target_env = "msvc", target_os = "illumos")))]
 #[global_allocator]
-static GLOBAL: MiMalloc = MiMalloc;
+static GLOBAL: Jemalloc = Jemalloc;
 
 use std::path::Path;
 use std::{env, thread};
 
-use humansize::{format_size, BINARY};
+use humansize::{format_size, FormatSizeOptions, BINARY};
 use owo_colors::OwoColorize;
 use rayon::prelude::*;
 use strum::IntoEnumIterator;
@@ -161,6 +181,29 @@ fn main() {
                 }
             }
         }
+        Mode::DumpSyntaxDot {
+            path,
+            ignore_comments,
+            language_overrides,
+        } => {
+            let path = Path::new(&path);
+            let bytes = read_or_die(path);
+            let src = String::from_utf8_lossy(&bytes).to_string();
+
+            let language = guess(path, &src, &language_overrides);
+            match language {
+                Some(lang) => {
+                    let ts_lang = tsp::from_language(lang);
+                    let arena = Arena::new();
+                    let ast = tsp::parse(&arena, &src, &ts_lang, ignore_comments);
+                    init_all_info(&ast, &[]);
+                    syntax::print_as_dot(&ast);
+                }
+                None => {
+                    eprintln!("No tree-sitter parser for file: {:?}", path);
+                }
+            }
+        }
         Mode::ListLanguages {
             use_color,
             language_overrides,
@@ -201,6 +244,7 @@ fn main() {
             display_options,
             set_exit_code,
             language_overrides,
+            binary_overrides,
         } => {
             let diff_result = diff_conflicts_file(
                 &display_path,
@@ -208,6 +252,7 @@ fn main() {
                 &display_options,
                 &diff_options,
                 &language_overrides,
+                &binary_overrides,
             );
 
             print_diff_result(&display_options, &diff_result);
@@ -224,6 +269,7 @@ fn main() {
             display_options,
             set_exit_code,
             language_overrides,
+            binary_overrides,
             lhs_path,
             rhs_path,
             lhs_permissions,
@@ -259,6 +305,7 @@ fn main() {
                         &display_options,
                         &diff_options,
                         &language_overrides,
+                        &binary_overrides,
                     );
 
                     if matches!(display_options.display_mode, DisplayMode::Json) {
@@ -313,6 +360,7 @@ fn main() {
                         &diff_options,
                         false,
                         &language_overrides,
+                        &binary_overrides,
                     );
                     if diff_result.has_reportable_change() {
                         encountered_changes = true;
@@ -351,10 +399,22 @@ fn diff_file(
     diff_options: &DiffOptions,
     missing_as_empty: bool,
     overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
+    binary_overrides: &[glob::Pattern],
 ) -> DiffResult {
     let (lhs_bytes, rhs_bytes) = read_files_or_die(lhs_path, rhs_path, missing_as_empty);
-    let (mut lhs_src, mut rhs_src) = match (guess_content(&lhs_bytes), guess_content(&rhs_bytes)) {
+
+    // Override here? Separate option or part of existing --override arg?
+
+    let (mut lhs_src, mut rhs_src) = match (
+        guess_content(&lhs_bytes, lhs_path, binary_overrides),
+        guess_content(&rhs_bytes, rhs_path, binary_overrides),
+    ) {
         (ProbableFileKind::Binary, _) | (_, ProbableFileKind::Binary) => {
+            let has_byte_changes = if lhs_bytes == rhs_bytes {
+                None
+            } else {
+                Some((lhs_bytes.len(), rhs_bytes.len()))
+            };
             return DiffResult {
                 extra_info: renamed,
                 display_path: display_path.to_owned(),
@@ -364,7 +424,7 @@ fn diff_file(
                 lhs_positions: vec![],
                 rhs_positions: vec![],
                 hunks: vec![],
-                has_byte_changes: lhs_bytes != rhs_bytes,
+                has_byte_changes,
                 has_syntactic_changes: false,
             };
         }
@@ -383,7 +443,7 @@ fn diff_file(
     // a trailing newline, e.g. "foo\n\bar\n" versus "foo". We want to
     // consider `foo` to be unchanged in this case.
     //
-    // Theoretically a tree-sitter parser coud change its AST due to
+    // Theoretically a tree-sitter parser could change its AST due to
     // the additional trailing newline, but it seems vanishingly
     // unlikely.
     if !lhs_src.is_empty() && !lhs_src.ends_with('\n') {
@@ -429,9 +489,10 @@ fn diff_conflicts_file(
     display_options: &DisplayOptions,
     diff_options: &DiffOptions,
     overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
+    binary_overrides: &[glob::Pattern],
 ) -> DiffResult {
     let bytes = read_file_or_die(path);
-    let mut src = match guess_content(&bytes) {
+    let mut src = match guess_content(&bytes, path, binary_overrides) {
         ProbableFileKind::Text(src) => src,
         ProbableFileKind::Binary => {
             eprintln!("error: Expected a text file with conflict markers, got a binary file.");
@@ -496,7 +557,11 @@ fn check_only_text(
     lhs_src: &str,
     rhs_src: &str,
 ) -> DiffResult {
-    let has_changes = lhs_src != rhs_src;
+    let has_byte_changes = if lhs_src == rhs_src {
+        None
+    } else {
+        Some((lhs_src.as_bytes().len(), rhs_src.as_bytes().len()))
+    };
 
     DiffResult {
         display_path: display_path.to_owned(),
@@ -507,8 +572,8 @@ fn check_only_text(
         lhs_positions: vec![],
         rhs_positions: vec![],
         hunks: vec![],
-        has_byte_changes: has_changes,
-        has_syntactic_changes: has_changes,
+        has_byte_changes,
+        has_syntactic_changes: lhs_src != rhs_src,
     }
 }
 
@@ -523,13 +588,12 @@ fn diff_file_content(
     diff_options: &DiffOptions,
     overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
 ) -> DiffResult {
-    let (guess_src, guess_path) = match rhs_path {
-        FileArgument::NamedPath(path) => (&rhs_src, Path::new(path)),
-        FileArgument::Stdin => (&rhs_src, Path::new(&display_path)),
-        FileArgument::DevNull => (&lhs_src, Path::new(&display_path)),
+    let guess_src = match rhs_path {
+        FileArgument::DevNull => &lhs_src,
+        _ => &rhs_src,
     };
 
-    let language = guess(guess_path, guess_src, overrides);
+    let language = guess(Path::new(display_path), guess_src, overrides);
     let lang_config = language.map(|lang| (lang, tsp::from_language(lang)));
 
     if lhs_src == rhs_src {
@@ -549,7 +613,7 @@ fn diff_file_content(
             lhs_positions: vec![],
             rhs_positions: vec![],
             hunks: vec![],
-            has_byte_changes: false,
+            has_byte_changes: None,
             has_syntactic_changes: false,
         };
     }
@@ -581,6 +645,13 @@ fn diff_file_content(
                         Ok((lhs, rhs)) => {
                             if diff_options.check_only {
                                 let has_syntactic_changes = lhs != rhs;
+
+                                let has_byte_changes = if lhs_src == rhs_src {
+                                    None
+                                } else {
+                                    Some((lhs_src.as_bytes().len(), rhs_src.as_bytes().len()))
+                                };
+
                                 return DiffResult {
                                     extra_info,
                                     display_path: display_path.to_owned(),
@@ -590,7 +661,7 @@ fn diff_file_content(
                                     lhs_positions: vec![],
                                     rhs_positions: vec![],
                                     hunks: vec![],
-                                    has_byte_changes: true,
+                                    has_byte_changes,
                                     has_syntactic_changes,
                                 };
                             }
@@ -683,10 +754,11 @@ fn diff_file_content(
                     }
                 }
                 Err(tsp::ExceededByteLimit(num_bytes)) => {
+                    let format_options = FormatSizeOptions::from(BINARY).decimal_places(1);
                     let file_format = FileFormat::TextFallback {
                         reason: format!(
                             "{} exceeded DFT_BYTE_LIMIT",
-                            &format_size(num_bytes, BINARY)
+                            &format_size(num_bytes, format_options)
                         ),
                     };
 
@@ -722,6 +794,12 @@ fn diff_file_content(
     );
     let has_syntactic_changes = !hunks.is_empty();
 
+    let has_byte_changes = if lhs_src == rhs_src {
+        None
+    } else {
+        Some((lhs_src.as_bytes().len(), rhs_src.as_bytes().len()))
+    };
+
     DiffResult {
         extra_info,
         display_path: display_path.to_owned(),
@@ -731,7 +809,7 @@ fn diff_file_content(
         lhs_positions,
         rhs_positions,
         hunks,
-        has_byte_changes: true,
+        has_byte_changes,
         has_syntactic_changes,
     }
 }
@@ -748,10 +826,12 @@ fn diff_directories<'a>(
     display_options: &DisplayOptions,
     diff_options: &DiffOptions,
     overrides: &[(LanguageOverride, Vec<glob::Pattern>)],
+    binary_overrides: &[glob::Pattern],
 ) -> impl ParallelIterator<Item = DiffResult> + 'a {
     let diff_options = diff_options.clone();
     let display_options = display_options.clone();
     let overrides: Vec<_> = overrides.into();
+    let binary_overrides: Vec<_> = binary_overrides.into();
 
     // We greedily list all files in the directory, and then diff them
     // in parallel. This is assuming that diffing is slower than
@@ -775,6 +855,7 @@ fn diff_directories<'a>(
             &diff_options,
             true,
             &overrides,
+            &binary_overrides,
         )
     })
 }
@@ -867,7 +948,7 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
             }
         }
         (FileContent::Binary, FileContent::Binary) => {
-            if display_options.print_unchanged || summary.has_byte_changes {
+            if display_options.print_unchanged || summary.has_byte_changes.is_some() {
                 println!(
                     "{}",
                     display::style::header(
@@ -879,10 +960,17 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         display_options
                     )
                 );
-                if summary.has_byte_changes {
-                    println!("Binary contents changed.\n");
-                } else {
-                    println!("No changes.\n");
+
+                match summary.has_byte_changes {
+                    Some((lhs_len, rhs_len)) => {
+                        let format_options = FormatSizeOptions::from(BINARY).decimal_places(1);
+                        println!(
+                            "Binary contents changed (old: {}, new: {}).\n",
+                            &format_size(lhs_len, format_options),
+                            &format_size(rhs_len, format_options),
+                        )
+                    }
+                    None => println!("No changes.\n"),
                 }
             }
         }
