@@ -8,9 +8,7 @@ use std::{
 };
 
 use clap::{crate_authors, crate_description, value_parser, Arg, ArgAction, Command};
-use const_format::formatcp;
 use crossterm::tty::IsTty;
-use itertools::Itertools;
 
 use crate::{
     display::style::BackgroundColor,
@@ -91,7 +89,7 @@ impl Default for DiffOptions {
     }
 }
 
-fn app() -> clap::Command<'static> {
+fn app() -> clap::Command {
     Command::new("Difftastic")
         .override_usage(USAGE)
         .version(env!("CARGO_PKG_VERSION"))
@@ -117,7 +115,6 @@ fn app() -> clap::Command<'static> {
         .arg(
             Arg::new("dump-syntax")
                 .long("dump-syntax")
-                .takes_value(true)
                 .value_name("PATH")
                 .action(ArgAction::Set)
                 .long_help(
@@ -127,7 +124,6 @@ fn app() -> clap::Command<'static> {
         .arg(
             Arg::new("dump-syntax-dot")
                 .long("dump-syntax-dot")
-                .takes_value(true)
                 .value_name("PATH")
                 .action(ArgAction::Set)
                 .long_help(
@@ -137,8 +133,7 @@ fn app() -> clap::Command<'static> {
         .arg(
             Arg::new("dump-ts")
                 .long("dump-ts")
-                .takes_value(true)
-                .value_name("PATH")
+                                .value_name("PATH")
                 .action(ArgAction::Set)
                 .long_help(
                     "Parse a single file with tree-sitter and display the tree-sitter parse tree.",
@@ -147,36 +142,33 @@ fn app() -> clap::Command<'static> {
         .arg(
             Arg::new("context")
                 .long("context")
-                .takes_value(true)
-                .value_name("LINES")
+                                .value_name("LINES")
                 .action(ArgAction::Set)
                 .long_help("The number of contextual lines to show around changed lines.")
                 .default_value("3")
                 .env("DFT_CONTEXT")
-                .validator(|s| s.parse::<u32>())
+                .value_parser(clap::value_parser!(u32))
                 .required(false),
         )
         .arg(
             Arg::new("width")
                 .long("width")
-                .takes_value(true)
                 .value_name("COLUMNS")
                 .action(ArgAction::Set)
                 .long_help("Use this many columns when calculating line wrapping. If not specified, difftastic will detect the terminal width.")
                 .env("DFT_WIDTH")
-                .validator(|s| s.parse::<usize>())
+                .value_parser(clap::value_parser!(usize))
                 .required(false),
         )
         .arg(
             Arg::new("tab-width")
                 .long("tab-width")
-                .takes_value(true)
                 .value_name("NUM_SPACES")
                 .action(ArgAction::Set)
                 .long_help("Treat a tab as this many spaces.")
                 .env("DFT_TAB_WIDTH")
-                .default_value(formatcp!("{}", DEFAULT_TAB_WIDTH))
-                .validator(|s| s.parse::<usize>())
+                .default_value(format!("{}", DEFAULT_TAB_WIDTH))
+                .value_parser(clap::value_parser!(usize))
                 .required(false),
         )
         .arg(
@@ -236,7 +228,7 @@ json: Output the results as a machine-readable JSON array with an element per fi
                 .value_parser(["on", "off"])
                 .default_value("on")
                 .action(ArgAction::Set)
-                .help("Remove any carriage return characters before diffing. This can be helpful when dealing with files on Windows that contain CRLF, i.e. `\\r\\n`.\n\nWhen disabled, difftastic will consider multiline string literals (in code) or mutiline text (e.g. in HTML) to differ if the two input files have different line endings.")
+                .help("Remove any carriage return characters before diffing. This can be helpful when dealing with files on Windows that contain CRLF, i.e. `\\r\\n`.\n\nWhen disabled, difftastic will consider multiline string literals (in code) or multiline text (e.g. in HTML) to differ if the two input files have different line endings.")
         )
         .arg(
             Arg::new("check-only").long("check-only")
@@ -254,11 +246,7 @@ json: Output the results as a machine-readable JSON array with an element per fi
             Arg::new("skip-unchanged").long("skip-unchanged")
                 .action(ArgAction::SetTrue)
                 .env("DFT_SKIP_UNCHANGED")
-                .help("Don't display anything if a file is unchanged.")
-        )
-        .arg(
-            Arg::new("missing-as-empty").long("missing-as-empty")
-                .help("Treat paths that don't exist as equivalent to an empty file. Only applies when diffing files, not directories.")
+                .help("Don't display anything if a file is unchanged. This is useful when comparing directories of files.")
         )
         .arg(
             Arg::new("override").long("override")
@@ -283,49 +271,66 @@ $ export DFT_OVERRIDE_2='*.js:javascript jsx'
 When multiple overrides are specified, the first matching override wins."))
                 .env("DFT_OVERRIDE")
         )
+        .arg(
+            Arg::new("override-binary").long("override-binary")
+                .value_name("GLOB")
+                .action(ArgAction::Append)
+                .help(concat!("Treat file names matching this glob as binary files, overriding normal binary detection. For example:
+
+$ ", env!("CARGO_BIN_NAME"), " --override-binary='*.gz' old.gz new.gz
+
+This argument may be given more than once. For example:
+
+$ ", env!("CARGO_BIN_NAME"), " --override-binary='*.gz' --override-binary='foo.pickle' old.gz new.gz
+
+To configure multiple overrides using environment variables, difftastic also accepts DFT_OVERRIDE_BINARY_1 up to DFT_OVERRIDE_BINARY_9.
+
+$ export DFT_OVERRIDE_BINARY='*.gz'
+$ export DFT_OVERRIDE_BINARY_1='*.bz2'
+$ export DFT_OVERRIDE_BINARY_2='foo.pickle'"))
+                .env("DFT_OVERRIDE_BINARY")
+        )
         .arg(
             Arg::new("list-languages").long("list-languages")
                 .action(ArgAction::SetTrue)
-                .help("Print all the languages supported by difftastic, along with their extensions.")
+                .help("Print all the languages supported by difftastic, along with their recognised extensions.")
         )
         .arg(
             Arg::new("byte-limit").long("byte-limit")
-                .takes_value(true)
                 .value_name("LIMIT")
                 .action(ArgAction::Set)
-                .help("Use a text diff if either input file exceeds this size.")
-                .default_value(formatcp!("{}", DEFAULT_BYTE_LIMIT))
+                .help("Use a line-oriented diff if either input file exceeds this size.")
+                .default_value(format!("{}", DEFAULT_BYTE_LIMIT))
                 .env("DFT_BYTE_LIMIT")
-                .validator(|s| s.parse::<usize>())
+                .value_parser(clap::value_parser!(usize))
                 .required(false),
         )
         .arg(
             Arg::new("graph-limit").long("graph-limit")
-                .takes_value(true)
                 .value_name("LIMIT")
-                .help("Use a text diff if the structural graph exceed this number of nodes in memory.")
-                .default_value(formatcp!("{}", DEFAULT_GRAPH_LIMIT))
+                .help("Use a line-oriented diff if the internal graph exceeds this number of vertices. This limit controls the worst case runtime and memory usage for difftastic.
+
+Higher values will allow difftastic to perform a structural diff in more cases. Higher values will also increase the time before difftastic gives up on structural diffing, and increase peak memory usage.")
+                .default_value(format!("{}", DEFAULT_GRAPH_LIMIT))
                 .action(ArgAction::Set)
                 .env("DFT_GRAPH_LIMIT")
-                .validator(|s| s.parse::<usize>())
+                .value_parser(clap::value_parser!(usize))
                 .required(false),
         )
         .arg(
             Arg::new("parse-error-limit").long("parse-error-limit")
-                .takes_value(true)
                 .value_name("LIMIT")
                 .action(ArgAction::Set)
-                .help("Use a text diff if the number of parse errors exceeds this value.")
-                .default_value(formatcp!("{}", DEFAULT_PARSE_ERROR_LIMIT))
+                .help("Use a line-oriented diff if the number of parse errors exceeds this value.")
+                .default_value(format!("{}", DEFAULT_PARSE_ERROR_LIMIT))
                 .env("DFT_PARSE_ERROR_LIMIT")
-                .validator(|s| s.parse::<usize>())
+                .value_parser(clap::value_parser!(usize))
                 .required(false),
         )
         .arg(
             Arg::new("paths")
                 .value_name("PATHS")
                 .action(ArgAction::Append)
-                .multiple_values(true)
                 .hide(true)
                 .value_parser(value_parser!(OsString)),
         )
@@ -480,6 +485,7 @@ pub(crate) enum Mode {
         display_options: DisplayOptions,
         set_exit_code: bool,
         language_overrides: Vec<(LanguageOverride, Vec<glob::Pattern>)>,
+        binary_overrides: Vec<glob::Pattern>,
         /// The path where we can read the LHS file. This is often a
         /// temporary file generated by source control.
         lhs_path: FileArgument,
@@ -498,6 +504,7 @@ pub(crate) enum Mode {
         display_options: DisplayOptions,
         set_exit_code: bool,
         language_overrides: Vec<(LanguageOverride, Vec<glob::Pattern>)>,
+        binary_overrides: Vec<glob::Pattern>,
         path: FileArgument,
         /// The path that we show to the user.
         display_path: String,
@@ -627,26 +634,55 @@ fn parse_overrides_or_die(raw_overrides: &[String]) -> Vec<(LanguageOverride, Ve
         std::process::exit(EXIT_BAD_ARGUMENTS);
     }
 
+    let mut combined_overrides: Vec<(LanguageOverride, Vec<glob::Pattern>)> = vec![];
+    for (lang, globs) in overrides {
+        if let Some((prev_lang, prev_globs)) = combined_overrides.last_mut() {
+            if *prev_lang == lang {
+                prev_globs.extend(globs);
+            } else {
+                combined_overrides.push((lang, globs));
+            }
+        } else {
+            combined_overrides.push((lang, globs));
+        }
+    }
+
+    combined_overrides
+}
+
+fn parse_binary_overrides_or_die(glob_strs: &[String]) -> Vec<glob::Pattern> {
+    let mut overrides: Vec<glob::Pattern> = vec![];
+    let mut invalid_syntax = false;
+
+    for glob_str in glob_strs {
+        match glob::Pattern::new(glob_str) {
+            Ok(pattern) => {
+                overrides.push(pattern);
+            }
+            Err(e) => {
+                eprintln!("Invalid glob syntax '{}'", glob_str);
+                eprintln!("Glob parsing error: {}", e.msg);
+                invalid_syntax = true;
+            }
+        }
+    }
+
+    if invalid_syntax {
+        std::process::exit(EXIT_BAD_ARGUMENTS);
+    }
+
     overrides
-        .into_iter()
-        .coalesce(
-            |(prev_lang, mut prev_globs), (current_lang, current_globs)| {
-                if prev_lang == current_lang {
-                    prev_globs.extend(current_globs);
-                    Ok((prev_lang, prev_globs))
-                } else {
-                    Err(((prev_lang, prev_globs), (current_lang, current_globs)))
-                }
-            },
-        )
-        .collect()
 }
 
 /// Parse CLI arguments passed to the binary.
 pub(crate) fn parse_args() -> Mode {
     let matches = app().get_matches();
 
-    let color_output = match matches.value_of("color").expect("color has a default") {
+    let color_output = match matches
+        .get_one::<String>("color")
+        .map(|s| s.as_str())
+        .expect("color has a default")
+    {
         "always" => ColorOutput::Always,
         "never" => ColorOutput::Never,
         "auto" => ColorOutput::Auto,
@@ -658,9 +694,21 @@ pub(crate) fn parse_args() -> Mode {
 
     let ignore_comments = matches.get_flag("ignore-comments");
 
+    let mut raw_binary_overrides: Vec<String> = vec![];
+    if let Some(binary_overrides) = matches.get_many("override-binary") {
+        raw_binary_overrides = binary_overrides.cloned().collect();
+    }
+    for i in 1..=9 {
+        if let Ok(value) = env::var(format!("DFT_OVERRIDE_BINARY_{}", i)) {
+            raw_binary_overrides.push(value);
+        }
+    }
+
+    let binary_overrides = parse_binary_overrides_or_die(&raw_binary_overrides);
+
     let mut raw_overrides: Vec<String> = vec![];
-    if let Some(overrides) = matches.values_of("override") {
-        raw_overrides = overrides.map(|s| s.into()).collect();
+    if let Some(overrides) = matches.get_many("override") {
+        raw_overrides = overrides.cloned().collect();
     }
     for i in 1..=9 {
         if let Ok(value) = env::var(format!("DFT_OVERRIDE_{}", i)) {
@@ -677,7 +725,7 @@ pub(crate) fn parse_args() -> Mode {
         };
     }
 
-    if let Some(path) = matches.value_of("dump-syntax") {
+    if let Some(path) = matches.get_one::<String>("dump-syntax") {
         return Mode::DumpSyntax {
             path: path.to_owned(),
             ignore_comments,
@@ -685,7 +733,7 @@ pub(crate) fn parse_args() -> Mode {
         };
     }
 
-    if let Some(path) = matches.value_of("dump-syntax-dot") {
+    if let Some(path) = matches.get_one::<String>("dump-syntax-dot") {
         return Mode::DumpSyntaxDot {
             path: path.to_owned(),
             ignore_comments,
@@ -693,22 +741,24 @@ pub(crate) fn parse_args() -> Mode {
         };
     }
 
-    if let Some(path) = matches.value_of("dump-ts") {
+    if let Some(path) = matches.get_one::<String>("dump-ts") {
         return Mode::DumpTreeSitter {
             path: path.to_owned(),
             language_overrides,
         };
     }
 
-    let terminal_width = if let Some(arg_width) = matches.value_of("width") {
-        arg_width
-            .parse::<usize>()
-            .expect("Already validated by clap")
+    let terminal_width = if let Some(arg_width) = matches.get_one::<usize>("width") {
+        *arg_width
     } else {
         detect_terminal_width()
     };
 
-    let display_mode = match matches.value_of("display").expect("display has a default") {
+    let display_mode = match matches
+        .get_one::<String>("display")
+        .map(|s| s.as_str())
+        .expect("display has a default")
+    {
         "side-by-side" => DisplayMode::SideBySide,
         "side-by-side-show-both" => DisplayMode::SideBySideShowBoth,
         "inline" => DisplayMode::Inline,
@@ -726,7 +776,8 @@ pub(crate) fn parse_args() -> Mode {
     };
 
     let background_color = match matches
-        .value_of("background")
+        .get_one::<String>("background")
+        .map(|s| s.as_str())
         .expect("Always present as we've given clap a default")
     {
         "dark" => BackgroundColor::Dark,
@@ -734,45 +785,38 @@ pub(crate) fn parse_args() -> Mode {
         _ => unreachable!("clap has already validated the values"),
     };
 
-    let syntax_highlight = matches.value_of("syntax-highlight") == Some("on");
+    let syntax_highlight = matches
+        .get_one::<String>("syntax-highlight")
+        .map(|s| s.as_str())
+        == Some("on");
 
     let sort_paths = matches.get_flag("sort-paths");
 
-    let graph_limit = matches
-        .value_of("graph-limit")
-        .expect("Always present as we've given clap a default")
-        .parse::<usize>()
-        .expect("Value already validated by clap");
+    let graph_limit = *matches
+        .get_one("graph-limit")
+        .expect("Always present as we've given clap a default");
 
-    let byte_limit = matches
-        .value_of("byte-limit")
-        .expect("Always present as we've given clap a default")
-        .parse::<usize>()
-        .expect("Value already validated by clap");
+    let byte_limit = *matches
+        .get_one("byte-limit")
+        .expect("Always present as we've given clap a default");
 
-    let parse_error_limit = matches
-        .value_of("parse-error-limit")
-        .expect("Always present as we've given clap a default")
-        .parse::<usize>()
-        .expect("Value already validated by clap");
+    let parse_error_limit = *matches
+        .get_one("parse-error-limit")
+        .expect("Always present as we've given clap a default");
 
-    let tab_width = matches
-        .value_of("tab-width")
-        .expect("Always present as we've given clap a default")
-        .parse::<usize>()
-        .expect("Value already validated by clap");
+    let tab_width = *matches
+        .get_one("tab-width")
+        .expect("Always present as we've given clap a default");
 
-    let num_context_lines = matches
-        .value_of("context")
-        .expect("Always present as we've given clap a default")
-        .parse::<u32>()
-        .expect("Value already validated by clap");
+    let num_context_lines = *matches
+        .get_one("context")
+        .expect("Always present as we've given clap a default");
 
     let print_unchanged = !matches.get_flag("skip-unchanged");
 
     let set_exit_code = matches.get_flag("exit-code");
 
-    let strip_cr = matches.value_of("strip-cr") == Some("on");
+    let strip_cr = matches.get_one::<String>("strip-cr").map(|s| s.as_str()) == Some("on");
 
     let check_only = matches.get_flag("check-only");
 
@@ -785,7 +829,10 @@ pub(crate) fn parse_args() -> Mode {
         strip_cr,
     };
 
-    let args: Vec<_> = matches.values_of_os("paths").unwrap_or_default().collect();
+    let args = matches
+        .get_raw("paths")
+        .unwrap_or_default()
+        .collect::<Vec<_>>();
     info!("CLI arguments: {:?}", args);
 
     // Print git environment variables so we can see the additional
@@ -869,6 +916,7 @@ pub(crate) fn parse_args() -> Mode {
                 display_options,
                 set_exit_code,
                 language_overrides,
+                binary_overrides,
             };
         }
         _ => {
@@ -902,6 +950,7 @@ pub(crate) fn parse_args() -> Mode {
         display_options,
         set_exit_code,
         language_overrides,
+        binary_overrides,
         lhs_path,
         rhs_path,
         lhs_permissions,
