@@ -13,9 +13,9 @@ use crate::parse::guess_language as guess;
 use crate::parse::syntax::{AtomKind, Syntax};
 
 /// A language may contain certain nodes that are in other languages
-/// and should be parsed as such (e.g. HTML `<script>` nodes containing
-/// JavaScript). This contains how to identify such nodes, and what
-/// languages we should parse them as.
+/// and should be parsed as such (e.g. HTML `<script>` nodes
+/// containing JavaScript). This struct describes how to identify such
+/// nodes, and what languages we should parse them as.
 ///
 /// Note that we don't support sub-languages more than one layer deep.
 pub(crate) struct TreeSitterSubLanguage {
@@ -32,17 +32,27 @@ pub(crate) struct TreeSitterConfig {
     /// The tree-sitter language parser.
     pub(crate) language: ts::Language,
 
-    /// Tree-sitter nodes that we treat as indivisible atoms.
+    /// Force these tree-sitter nodes to be difftastic atoms and
+    /// ignore their children.
     ///
-    /// This is particularly useful for strings, as some grammars use
-    /// several nodes for a single string literal. We don't want to
-    /// say e.g. the closing string delimiter moved, as it's confusing
-    /// and not well-balanced syntax.
+    /// Difftastic only cares about list delimiters and atom
+    /// contents. This ensures that `"x"` and `" x"` are different,
+    /// but `[x]` and `[ x]` are not.
     ///
-    /// This is also useful for when tree-sitter nodes don't include
-    /// all the children in the source. This is known limitation of
-    /// tree-sitter, and occurs more often for complex string syntax.
+    /// This causes problems for tree-sitter grammars that have more
+    /// complex structure for literals. If string interpolation
+    /// produces an AST with a separate interpolation node, difftastic
+    /// will think that `"$x"` and `" $x"` are the same, because the atom
+    /// is just `$x` and the delimiter is `"`.
+    ///
+    /// This problem also occurs when the tree-sitter AST is missing
+    /// some children. This is known limitation of tree-sitter, and
+    /// occurs more often for complex string syntax.
     /// <https://github.com/tree-sitter/tree-sitter/issues/1156>
+    ///
+    /// By forcing the tree-sitter subtree to be a difftastic atom, we
+    /// guarantee a correct diff, at the cost of losing some structure
+    /// in the tree-sitter AST.
     atom_nodes: DftHashSet<&'static str>,
 
     /// We want to consider delimiter tokens as part of lists, not
@@ -50,7 +60,7 @@ pub(crate) struct TreeSitterConfig {
     /// mark which token pairs we consider to be delimiters.
     delimiter_tokens: Vec<(&'static str, &'static str)>,
 
-    /// Tree-sitter query used for syntax highlighting this
+    /// The tree-sitter query used for syntax highlighting this
     /// language.
     highlight_query: ts::Query,
 
@@ -69,8 +79,6 @@ extern "C" {
     fn tree_sitter_elisp() -> ts::Language;
     fn tree_sitter_elm() -> ts::Language;
     fn tree_sitter_elvish() -> ts::Language;
-    fn tree_sitter_erlang() -> ts::Language;
-    fn tree_sitter_fsharp() -> ts::Language;
     fn tree_sitter_gleam() -> ts::Language;
     fn tree_sitter_hare() -> ts::Language;
     fn tree_sitter_hack() -> ts::Language;
@@ -79,7 +87,6 @@ extern "C" {
     fn tree_sitter_kotlin() -> ts::Language;
     fn tree_sitter_latex() -> ts::Language;
     fn tree_sitter_newick() -> ts::Language;
-    fn tree_sitter_pascal() -> ts::Language;
     fn tree_sitter_perl() -> ts::Language;
     fn tree_sitter_qmljs() -> ts::Language;
     fn tree_sitter_r() -> ts::Language;
@@ -89,9 +96,7 @@ extern "C" {
     fn tree_sitter_scss() -> ts::Language;
     fn tree_sitter_solidity() -> ts::Language;
     fn tree_sitter_sql() -> ts::Language;
-    fn tree_sitter_swift() -> ts::Language;
     fn tree_sitter_vhdl() -> ts::Language;
-    fn tree_sitter_zig() -> ts::Language;
 }
 
 // TODO: begin/end and object/end.
@@ -222,6 +227,7 @@ pub(crate) fn from_language(language: guess::Language) -> TreeSitterConfig {
         }
         CommonLisp => {
             let language = unsafe { tree_sitter_commonlisp() };
+
             TreeSitterConfig {
                 language: language.clone(),
                 atom_nodes: ["str_lit", "char_lit"].into_iter().collect(),
@@ -362,7 +368,9 @@ pub(crate) fn from_language(language: guess::Language) -> TreeSitterConfig {
             }
         }
         Erlang => {
-            let language = unsafe { tree_sitter_erlang() };
+            let language_fn = tree_sitter_erlang::LANGUAGE;
+            let language = tree_sitter::Language::new(language_fn);
+
             TreeSitterConfig {
                 language: language.clone(),
                 atom_nodes: [].into_iter().collect(),
@@ -376,16 +384,16 @@ pub(crate) fn from_language(language: guess::Language) -> TreeSitterConfig {
             }
         }
         FSharp => {
-            let language = unsafe { tree_sitter_fsharp() };
+            let language_fn = tree_sitter_fsharp::LANGUAGE_FSHARP;
+            let language = tree_sitter::Language::new(language_fn);
+
             TreeSitterConfig {
                 language: language.clone(),
                 atom_nodes: ["string", "triple_quoted_string"].into_iter().collect(),
                 delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
-                highlight_query: ts::Query::new(
-                    &language,
-                    include_str!("../../vendored_parsers/highlights/f-sharp.scm"),
-                )
-                .unwrap(),
+                highlight_query: ts::Query::new(&language, tree_sitter_fsharp::HIGHLIGHTS_QUERY)
+                    .unwrap(),
+
                 sub_languages: vec![],
             }
         }
@@ -640,9 +648,14 @@ pub(crate) fn from_language(language: guess::Language) -> TreeSitterConfig {
                 // structure of complex types within, but it beats
                 // ignoring nullable changes.
                 // https://github.com/Wilfred/difftastic/issues/411
-                atom_nodes: ["line_string_literal", "character_literal", "nullable_type"]
-                    .into_iter()
-                    .collect(),
+                atom_nodes: [
+                    "nullable_type",
+                    "string_literal",
+                    "line_string_literal",
+                    "character_literal",
+                ]
+                .into_iter()
+                .collect(),
                 delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]"), ("<", ">")]
                     .into_iter()
                     .collect(),
@@ -773,7 +786,9 @@ pub(crate) fn from_language(language: guess::Language) -> TreeSitterConfig {
             }
         }
         Pascal => {
-            let language = unsafe { tree_sitter_pascal() };
+            let language_fn = tree_sitter_pascal::LANGUAGE;
+            let language = tree_sitter::Language::new(language_fn);
+
             TreeSitterConfig {
                 language: language.clone(),
                 atom_nodes: [].into_iter().collect(),
@@ -902,15 +917,18 @@ pub(crate) fn from_language(language: guess::Language) -> TreeSitterConfig {
             }
         }
         Rust => {
-            let language_fn = tree_sitter_rust::LANGUAGE;
+            let language_fn = tree_sitter_rust_orchard::LANGUAGE;
             let language = tree_sitter::Language::new(language_fn);
 
             TreeSitterConfig {
                 language: language.clone(),
                 atom_nodes: ["char_literal", "string_literal"].into_iter().collect(),
                 delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("|", "|"), ("<", ">")],
-                highlight_query: ts::Query::new(&language, tree_sitter_rust::HIGHLIGHTS_QUERY)
-                    .unwrap(),
+                highlight_query: ts::Query::new(
+                    &language,
+                    tree_sitter_rust_orchard::HIGHLIGHTS_QUERY,
+                )
+                .unwrap(),
                 sub_languages: vec![],
             }
         }
@@ -1001,16 +1019,15 @@ pub(crate) fn from_language(language: guess::Language) -> TreeSitterConfig {
             }
         }
         Swift => {
-            let language = unsafe { tree_sitter_swift() };
+            let language_fn = tree_sitter_swift::LANGUAGE;
+            let language = tree_sitter::Language::new(language_fn);
+
             TreeSitterConfig {
                 language: language.clone(),
                 atom_nodes: ["line_string_literal"].into_iter().collect(),
                 delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")],
-                highlight_query: ts::Query::new(
-                    &language,
-                    include_str!("../../vendored_parsers/highlights/swift.scm"),
-                )
-                .unwrap(),
+                highlight_query: ts::Query::new(&language, tree_sitter_swift::HIGHLIGHTS_QUERY)
+                    .unwrap(),
                 sub_languages: vec![],
             }
         }
@@ -1095,6 +1112,21 @@ pub(crate) fn from_language(language: guess::Language) -> TreeSitterConfig {
                 sub_languages: vec![],
             }
         }
+        Verilog => {
+            let language_fn = tree_sitter_verilog::LANGUAGE;
+            let language = tree_sitter::Language::new(language_fn);
+            TreeSitterConfig {
+                language: language.clone(),
+                atom_nodes: ["integral_number"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("begin", "end")],
+                highlight_query: ts::Query::new(
+                    &language,
+                    include_str!("../../vendored_parsers/highlights/verilog.scm"),
+                )
+                .unwrap(),
+                sub_languages: vec![],
+            }
+        }
         Vhdl => {
             let language = unsafe { tree_sitter_vhdl() };
             TreeSitterConfig {
@@ -1110,20 +1142,17 @@ pub(crate) fn from_language(language: guess::Language) -> TreeSitterConfig {
             }
         }
         Zig => {
-            let language = unsafe { tree_sitter_zig() };
+            let language_fn = tree_sitter_zig::LANGUAGE;
+            let language = tree_sitter::Language::new(language_fn);
+
             TreeSitterConfig {
                 language: language.clone(),
-                atom_nodes: ["STRINGLITERALSINGLE", "BUILTINIDENTIFIER"]
-                    .into_iter()
-                    .collect(),
+                atom_nodes: ["string"].into_iter().collect(),
                 delimiter_tokens: vec![("{", "}"), ("[", "]"), ("(", ")")]
                     .into_iter()
                     .collect(),
-                highlight_query: ts::Query::new(
-                    &language,
-                    include_str!("../../vendored_parsers/highlights/zig.scm"),
-                )
-                .unwrap(),
+                highlight_query: ts::Query::new(&language, tree_sitter_zig::HIGHLIGHTS_QUERY)
+                    .unwrap(),
                 sub_languages: vec![],
             }
         }
@@ -1797,7 +1826,12 @@ fn atom_from_cursor<'a>(
         AtomKind::Normal
     };
 
-    Some(Syntax::new_atom(arena, position, content, highlight))
+    Some(Syntax::new_atom(
+        arena,
+        position,
+        content.to_owned(),
+        highlight,
+    ))
 }
 
 #[cfg(test)]
