@@ -61,6 +61,17 @@ fn binary_changed() {
     cmd.assert().stdout(predicate_fn);
 }
 
+#[test]
+fn binary_override() {
+    let mut cmd = get_base_command();
+
+    cmd.arg("--override-binary=*.js")
+        .arg("sample_files/simple_1.js")
+        .arg("sample_files/simple_2.js");
+    let predicate_fn = predicate::str::contains("Binary contents changed");
+    cmd.assert().stdout(predicate_fn);
+}
+
 #[test]
 fn has_changes_default_exit_code() {
     let mut cmd = get_base_command();
@@ -116,6 +127,16 @@ fn check_only_text_file() {
     cmd.assert().stdout(predicate_fn);
 }
 
+#[test]
+fn text_changes_at_end_doesnt_crash() {
+    let mut cmd = get_base_command();
+
+    cmd.arg("sample_files/cli_tests/changes_at_end_1.txt")
+        .arg("sample_files/cli_tests/changes_at_end_2.txt");
+
+    cmd.assert().success();
+}
+
 #[test]
 fn makefile_text_as_atom() {
     let mut cmd = get_base_command();
